package idiomdemo.service;

import java.io.StringReader;
import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import org.jdom.Document;
import org.jdom.Element;
import org.jdom.input.SAXBuilder;
import org.jdom.output.XMLOutputter;

import com.idiomsoftware.decisionserver.DecisionServer;
import com.idiomsoftware.decisionserver.LocalDecisionServer;

public class CalculationServiceImpl implements CalculationService {

	private static DecisionServer decisionServer  = null;
	
	static {
		decisionServer = LocalDecisionServer.getInstance();
	}
	
	public String getServerTime() {
		return String.valueOf(System.currentTimeMillis());
	}

	public Double calculateBMI(Double height, Double weight) 
	{
		Double heightPowered = Math.pow(height / 100.00, 2);
		
		Double result = weight / heightPowered;
		
		BigDecimal bd = new BigDecimal(result);
		bd = bd.setScale(2, BigDecimal.ROUND_DOWN);
		return bd.doubleValue();
	}
	
	@SuppressWarnings("unchecked")
	public Double calculateBMIWithRules(Double height, Double weight) 
	{
		SAXBuilder saxBuilder = new SAXBuilder();
		List<Document> documents = new ArrayList<Document>();
		Document documentIn = new Document();
		XMLOutputter outputter = new XMLOutputter();
		
		String xmlInput = String.format("<BMI><Weight>%f</Weight><Height>%f</Height><CalculatedBMI>0</CalculatedBMI></BMI>", weight, (height / 100));
		
		try {
			documentIn = saxBuilder.build(new StringReader(xmlInput));
			documents.add(documentIn);
			System.out.println(outputter.outputString(documentIn));
			
			List listOut = decisionServer.executeDecisionsV3(new Date(), documents, "BMI", "CalculateBMI", null);
			Document docOut = (Document)listOut.get(0);
			
			Element element = docOut.getRootElement().getChild("CalculatedBMI");
			System.out.println(outputter.outputString(docOut));
			System.out.println("element value : " + element.getText());

			return new Double(element.getText());
			
		} catch (Exception e) {
			e.printStackTrace();
			throw new RuntimeException(e);
		}
	}
}
